import abc

import typing

from .experiment import Experiment


class ExperimentCollectionABC(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def close(self):
        pass

    @abc.abstractmethod
    def add_experiment(self, exp: Experiment):
        pass

    @abc.abstractmethod
    def delete_experiment(self, exp: typing.Union[Experiment, str]):
        pass

    @abc.abstractmethod
    def check_experiment(self, exp: typing.Union[Experiment, str]) -> bool:
        pass

    @abc.abstractmethod
    def get_experiments(self, normalize=True):
        pass



import datetime
import json

from .utils import prepare_dict


class Experiment:
    def __init__(self, name, params=None, metrics=None):
        self.params = {}
        self.metrics = {}
        self.name = name
        self.set_params({} if params is None else params)
        self.set_metrics({} if metrics is None else metrics)
        self.time = datetime.datetime.now()

    def set_params(self, params):
        self.params = prepare_dict(params)

    def set_metrics(self, metrics):
        self.metrics = prepare_dict(metrics)

    def dumps(self):
        p = json.dumps(self.params)
        m = json.dumps(self.metrics)
        t = self.time.isoformat()
        return self.name, p, m, t

    def dumps_json(self):
        return {
            'name': self.name,
            'params': json.dumps(self.params),
            'metrics': json.dumps(self.metrics),
            'time': self.time.isoformat()
        }



from .collection_abc import ExperimentCollectionABC
from .collection_file import ExperimentCollectionLocal
from .collection_remote import ExperimentCollectionRemote
from .experiment import Experiment

__all__ = ["Experiment", "ExperimentCollectionLocal", "ExperimentCollectionRemote", "ExperimentCollectionABC"]



import sqlite3

from flask import g, current_app

INIT_STATEMENT = """CREATE TABLE IF NOT EXISTS experiments
(
    collection_name text,
    name            text,
    params          text not null,
    metrics         text not null,
    created_at      text not null,
    PRIMARY KEY (collection_name, name)
)"""

INSERT_STATEMENT = """INSERT INTO experiments (collection_name, name, params, metrics, created_at)
VALUES (?, ?, ?, ?, ?)
"""

SELECT_STATEMENT = """SELECT name, params, metrics, created_at as time
FROM experiments
WHERE collection_name = ?"""

DELETE_STATEMENT = """DELETE
FROM experiments
WHERE collection_name = ?
  AND name = ?"""

SELECT_EXISTS_STATEMENT = """SELECT 1
FROM experiments
WHERE collection_name = ?
  AND name = ?"""


class ExperimentDB:
    def __init__(self, path='main.db'):
        self.conn = sqlite3.connect(path)
        with self.conn:
            self.conn.execute(INIT_STATEMENT)

    def close(self):
        self.conn.close()

    def add_experiment(self, collection_name: str, name: str, params: str, metrics: str, time: str):
        with self.conn:
            self.conn.execute(INSERT_STATEMENT, (collection_name, name, params, metrics, time,))

    def delete_experiment(self, collection_name: str, name: str):
        with self.conn:
            self.conn.execute(DELETE_STATEMENT, (collection_name, name,)).fetchone()

    def check_experiment(self, collection_name: str, name: str) -> bool:
        return self.conn.execute(SELECT_EXISTS_STATEMENT, (collection_name, name,)).fetchone() is not None

    def get_experiments(self, collection_name: str):
        return self.conn.execute(SELECT_STATEMENT, (collection_name,)).fetchall()


def get_db():
    if 'db' not in g:
        g.db = ExperimentDB(current_app.config['DB_PATH'])

    return g.db


__all__ = ['ExperimentDB', 'get_db', ]



from flask import Blueprint, jsonify, request

from experiment_collection.db import get_db

bp = Blueprint('main', __name__)


@bp.route('/experiment', methods=('GET', 'POST', 'DELETE'))
def experiment_view():
    collection_name = request.form['collection_name']
    name = request.form['name']
    db = get_db()
    if request.method == 'GET':
        js = db.check_experiment(collection_name, name)
    elif request.method == 'POST':
        params = request.form['params']
        metrics = request.form['metrics']
        time = request.form['time']
        js = db.add_experiment(collection_name, name, params, metrics, time)
    elif request.method == 'DELETE':
        js = db.delete_experiment(collection_name, name)
    else:
        raise
    return jsonify(js)


@bp.route('/all_experiments', methods=('POST',))
def all_experiments_view():
    collection_name = request.form['collection_name']
    db = get_db()
    js = db.get_experiments(collection_name)
    return jsonify(js)



import os

from flask import Flask


def create_app(test_config=None):
    app = Flask(__name__, instance_relative_config=True)
    app.config.from_pyfile('config.cfg', silent=False)

    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    from . import views
    app.register_blueprint(views.bp)

    return app



from pkg_resources import parse_requirements
from setuptools import find_packages, setup


def load_requirements(filename: str) -> list:
    requirements = []
    with open(filename, 'r') as f:
        for requirement in parse_requirements(f.read()):
            extras = '[{}]'.format(','.join(requirement.extras)) if requirement.extras else ''
            requirements.append(
                '{}{}{}'.format(requirement.name, extras, requirement.specifier)
            )
    return requirements


module_name = 'experiment_collection'

with open('README.md', 'rt') as f:
    long_description = f.read()

setup(
    name=module_name,
    version='0.1.5',
    description='Experiment collection',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/AsciiShell/experiment_collection',
    author='AsciiShell (Aleksey Podchezertsev)',
    author_email='dev@asciishell.ru',
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
    ],
    keywords=['python3'],
    packages=find_packages(exclude=['tests']),
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            '{0} = {0}.__main__:main'.format(module_name),
        ]
    },
    include_package_data=True,
    zip_safe=False,
    install_requires=load_requirements('requirements.txt'),
)



import json

import numpy as np
import pandas as pd


def prepare_dict(d: dict) -> dict:
    result = {}
    for k, v in d.items():
        if isinstance(v, (float, np.floating)):
            v = float(v)
        elif isinstance(v, dict):
            v = prepare_dict(v)
        result[k] = v
    return result


def normalize_col(source: pd.DataFrame, name: str) -> pd.DataFrame:
    s = source[name].map(json.loads)
    df = pd.json_normalize(s)
    df.columns = name + '_' + df.columns
    return df


def postprocess_df(df: pd.DataFrame, normalize: bool) -> pd.DataFrame:
    df['time'] = pd.to_datetime(df['time'])
    if normalize:
        df = pd.concat([
            df,
            normalize_col(df, 'params'),
            normalize_col(df, 'metrics'),
        ], axis=1)
        df.drop(columns=['params', 'metrics'], inplace=True)
    return df



import json
import typing
import urllib.parse

import pandas as pd
import requests

from .collection_abc import ExperimentCollectionABC
from .experiment import Experiment
from .utils import postprocess_df

INSERT_URL = 'experiment'
SELECT_EXISTS_URL = INSERT_URL
DELETE_URL = INSERT_URL
SELECT_URL = 'all_experiments'


class ExperimentCollectionRemoteException(Exception):
    pass


class ExperimentCollectionRemote(ExperimentCollectionABC):
    def __init__(self, host, collection_name):
        self.host = host
        self.collection_name = collection_name

    def close(self):
        pass

    def add_experiment(self, exp: Experiment):
        data = exp.dumps_json()
        data['collection_name'] = self.collection_name
        url = urllib.parse.urljoin(self.host, INSERT_URL)
        r = requests.post(url, data=data)
        if not r.ok:
            raise ExperimentCollectionRemoteException(r.text)

    def delete_experiment(self, exp: typing.Union[Experiment, str]):
        if isinstance(exp, Experiment):
            exp = exp.name
        data = {'collection_name': self.collection_name, 'name': exp}
        url = urllib.parse.urljoin(self.host, DELETE_URL)
        r = requests.delete(url, data=data)
        if not r.ok:
            raise ExperimentCollectionRemoteException(r.text)

    def check_experiment(self, exp: typing.Union[Experiment, str]) -> bool:
        if isinstance(exp, Experiment):
            exp = exp.name
        data = {'collection_name': self.collection_name, 'name': exp}
        url = urllib.parse.urljoin(self.host, SELECT_EXISTS_URL)
        r = requests.get(url, data=data)
        if not r.ok:
            raise ExperimentCollectionRemoteException(r.text)
        return json.loads(r.text)

    def get_experiments(self, normalize=True):
        data = {'collection_name': self.collection_name}
        url = urllib.parse.urljoin(self.host, SELECT_URL)
        r = requests.post(url, data=data)
        if not r.ok:
            raise ExperimentCollectionRemoteException(r.text)
        js = json.loads(r.text)

        df = pd.DataFrame(js, columns=['name', 'params', 'metrics', 'time'])
        df = postprocess_df(df, normalize)
        return df



import sqlite3
import typing

import pandas as pd

from .collection_abc import ExperimentCollectionABC
from .experiment import Experiment
from .utils import postprocess_df

INIT_STATEMENT = """CREATE TABLE IF NOT EXISTS experiments
(
    name          text primary key,
    params        text not null,
    metrics       text not null,
    created_at    text not null
)"""

INSERT_STATEMENT = """INSERT INTO experiments (name, params, metrics, created_at)
VALUES (?, ?, ?, ?)
"""

SELECT_STATEMENT = """SELECT name, params, metrics, created_at as time
FROM experiments"""

DELETE_STATEMENT = """DELETE
FROM experiments
WHERE name = ?"""

SELECT_EXISTS_STATEMENT = """SELECT 1
FROM experiments
WHERE  name = ?"""


class ExperimentCollectionLocal(ExperimentCollectionABC):
    def __init__(self, path='main.db'):
        self.conn = sqlite3.connect(path)
        with self.conn:
            self.conn.execute(INIT_STATEMENT)

    def close(self):
        self.conn.close()

    def add_experiment(self, exp: Experiment):
        data = exp.dumps()
        with self.conn:
            self.conn.execute(INSERT_STATEMENT, data)

    def delete_experiment(self, exp: typing.Union[Experiment, str]):
        if isinstance(exp, Experiment):
            exp = exp.name
        with self.conn:
            self.conn.execute(DELETE_STATEMENT, (exp,))

    def check_experiment(self, exp: typing.Union[Experiment, str]) -> bool:
        if isinstance(exp, Experiment):
            exp = exp.name
        return self.conn.execute(SELECT_EXISTS_STATEMENT, (exp,)).fetchone() is not None

    def get_experiments(self, normalize=True):
        df = pd.read_sql(SELECT_STATEMENT, self.conn)
        df = postprocess_df(df, normalize)
        return df
